using System;

namespace Props.Shop.Framework
{
    public class Filters
    {
        public Currency Currency { get; set; } = Currency.CAD;
        public float MinRating { get; set; } = 0.8f;
        public bool KeepUnrated { get; set; } = true;
        public bool EnableUpperPrice { get; set; } = false;
        private int upperPrice;

        public int UpperPrice
        {
            get
            {
                return upperPrice;
            }
            set
            {
                if (EnableUpperPrice) upperPrice = value;
            }
        }
        public int LowerPrice { get; set; }
        public int MinPurchases { get; set; }
        public bool KeepUnknownPurchaseCount { get; set; } = true;
        public int MinReviews { get; set; }
        public bool KeepUnknownRatingCount { get; set; } = true;
        public bool EnableMaxShippingFee { get; set; }
        private int maxShippingFee;

        public int MaxShippingFee
        {
            get
            {
                return maxShippingFee;
            }
            set
            {
                if (EnableMaxShippingFee) maxShippingFee = value;
            }
        }
        public bool KeepUnknownShipping { get; set; } = true;

        public override bool Equals(object obj)
        {
            if (obj == null || GetType() != obj.GetType())
            {
                return false;
            }
            Filters other = (Filters)obj;
            return
                Currency == other.Currency &&
                MinRating == other.MinRating &&
                KeepUnrated == other.KeepUnrated &&
                EnableUpperPrice == other.EnableUpperPrice &&
                UpperPrice == other.UpperPrice &&
                LowerPrice == other.LowerPrice &&
                MinPurchases == other.MinPurchases &&
                KeepUnknownPurchaseCount == other.KeepUnknownPurchaseCount &&
                MinReviews == other.MinReviews &&
                KeepUnknownRatingCount == other.KeepUnknownRatingCount &&
                EnableMaxShippingFee == other.EnableMaxShippingFee &&
                MaxShippingFee == other.MaxShippingFee &&
                KeepUnknownShipping == other.KeepUnknownShipping;

        }

        public override int GetHashCode()
        {
            return HashCode.Combine(
                Currency,
                MinRating,
                UpperPrice,
                LowerPrice,
                MinPurchases,
                MinReviews,
                MaxShippingFee);
        }
        public Filters Copy()
        {
            return (Filters)this.MemberwiseClone();
        }
    }
}